@@ -3,7 +3,7 @@
    Definitions for dhcpd... */
 
 /*
- * Copyright (c) 2004-2005 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2007 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -173,7 +173,7 @@ struct option_tag {
 
 /* An agent option structure.   We need a special structure for the
    Relay Agent Information option because if more than one appears in
-   a message, we have to keep them seperate. */
+   a message, we have to keep them separate. */
 
 struct agent_options {
 	struct agent_options *next;
@@ -187,6 +187,9 @@ struct option_cache {
 	struct expression *expression;
 	struct option *option;
 	struct data_string data;
+
+	#define OPTION_HAD_NULLS	0x00000001
+	u_int32_t flags;
 };
 
 struct option_state {
@@ -232,6 +235,12 @@ struct packet {
 
 	int known;
 	int authenticated;
+
+	/* If we stash agent options onto the packet option state, to pretend
+	 * options we got in a previous exchange were still there, we need
+	 * to signal this in a reliable way.
+	 */
+	isc_boolean_t agent_options_stashed;
 };
 
 /* A network interface's MAC address. */
@@ -311,8 +320,15 @@ struct lease {
 	
 	struct lease_state *state;
 
+	/* 'tsfp' is more of an 'effective' tsfp.  It may be calculated from
+	 * stos+mclt for example if it's an expired lease and the server is
+	 * in partner-down state.  'atsfp' is zeroed whenever a lease is
+	 * updated - and only set when the peer acknowledges it.  This
+	 * ensures every state change is transmitted.
+	 */
 	TIME tstp;	/* Time sent to partner. */
 	TIME tsfp;	/* Time sent from partner. */
+	TIME atsfp;	/* Actual time sent from partner. */
 	TIME cltt;	/* Client last transaction time. */
 	struct lease *next_pending;
 };
@@ -329,7 +345,7 @@ struct lease_state {
 	struct option_state *options;
 	struct data_string parameter_request_list;
 	int max_message_size;
-	u_int32_t expiry, renewal, rebind;
+	unsigned char expiry[4], renewal[4], rebind[4];
 	struct data_string filename, server_name;
 	int got_requested_address;
 	int got_server_identifier;
@@ -421,7 +437,7 @@ struct lease_state {
 #endif
 
 #if !defined (DEFAULT_MIN_LEASE_TIME)
-# define DEFAULT_MIN_LEASE_TIME 0
+# define DEFAULT_MIN_LEASE_TIME 300
 #endif
 
 #if !defined (DEFAULT_MAX_LEASE_TIME)
@@ -978,6 +994,8 @@ int store_options PROTO ((int *, unsigned char *, unsigned, struct packet *,
 			  struct option_state *, struct binding_scope **,
 			  unsigned *, int, unsigned, unsigned,
 			  int, const char *));
+int format_has_text(const char *);
+int format_min_length(const char *, struct option_cache *);
 const char *pretty_print_option PROTO ((struct option *, const unsigned char *,
 					unsigned, int, int));
 int get_option (struct data_string *, struct universe *,
@@ -1753,7 +1771,7 @@ int if_readsocket PROTO ((omapi_object_t *));
 void reinitialize_interfaces PROTO ((void));
 
 /* dispatch.c */
-void set_time (u_int32_t);
+void set_time(TIME);
 struct timeval *process_outstanding_timeouts (struct timeval *);
 void dispatch PROTO ((void));
 isc_result_t got_one PROTO ((omapi_object_t *));
@@ -1928,7 +1946,7 @@ ssize_t decode_hw_header PROTO ((struct interface_info *, unsigned char *,
 				 unsigned, struct hardware *));
 ssize_t decode_udp_ip_header PROTO ((struct interface_info *, unsigned char *,
 				     unsigned, struct sockaddr_in *,
-				     unsigned));
+				     unsigned, unsigned *));
 
 /* ethernet.c */
 void assemble_ethernet_header PROTO ((struct interface_info *, unsigned char *,
