@@ -3,7 +3,7 @@
    Functions supporting the object management protocol... */
 
 /*
- * Copyright (c) 2004-2008 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2006 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1999-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -22,19 +22,19 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   https://www.isc.org/
+ *   http://www.isc.org/
  *
  * This software has been written for Internet Systems Consortium
  * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
  * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
+ * ``http://www.isc.org/''.  To learn more about Vixie Enterprises,
  * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
  * ``http://www.nominum.com''.
  */
 
 #ifndef lint
 static char ocopyright[] =
-"$Id: protocol.c,v 1.30.116.5 2009/07/23 21:43:35 sar Exp $ Copyright 2004-2008 Internet Systems Consortium.";
+"$Id: protocol.c,v 1.25.2.11 2006/02/22 22:43:27 dhankins Exp $ Copyright 2004-2006 Internet Systems Consortium.";
 #endif
 
 #include <omapip/omapip_p.h>
@@ -150,10 +150,6 @@ isc_result_t omapi_protocol_send_intro (omapi_object_t *h,
 	return ISC_R_SUCCESS;
 }
 
-#ifdef DEBUG_PROTOCOL
-extern const char *omapi_message_op_name(int);
-#endif /* DEBUG_PROTOCOL */
-
 isc_result_t omapi_protocol_send_message (omapi_object_t *po,
 					  omapi_object_t *id,
 					  omapi_object_t *mo,
@@ -180,9 +176,9 @@ isc_result_t omapi_protocol_send_message (omapi_object_t *po,
 	om = (omapi_message_object_t *)omo;
 
 #ifdef DEBUG_PROTOCOL
-	log_debug ("omapi_protocol_send_message(): "
-		   "op=%s  handle=%#lx  id=%#lx  rid=%#lx",
-		   omapi_message_op_name (m->op),
+	log_debug ("omapi_protocol_send_message()"
+		   "op=%ld  handle=%#lx  id=%#lx  rid=%#lx",
+		   (long)m -> op,
 		   (long)(m -> object ? m -> object -> handle : m -> handle),
 		   (long)p -> next_xid, (long)m -> rid);
 #endif
@@ -414,7 +410,7 @@ isc_result_t omapi_protocol_signal_handler (omapi_object_t *h,
 	    dmalloc_dump_outstanding ();
 #endif
 #if defined (DEBUG_RC_HISTORY_EXHAUSTIVELY)
-	    dump_rc_history (h);
+	    dump_rc_history ();
 #endif
 	    for (m = omapi_registered_messages; m; m = m -> next) {
 		if (m -> protocol_object == p) {
@@ -422,9 +418,6 @@ isc_result_t omapi_protocol_signal_handler (omapi_object_t *h,
 			omapi_signal (m -> object, "disconnect");
 		}
 	    }
-
-	    /* XXX */
-	    return ISC_R_SUCCESS;
 	}
 
 	/* Not a signal we recognize? */
@@ -499,7 +492,7 @@ isc_result_t omapi_protocol_signal_handler (omapi_object_t *h,
 			dmalloc_dump_outstanding ();
 #endif
 #if defined (DEBUG_RC_HISTORY_EXHAUSTIVELY)
-			dump_rc_history (h);
+			dump_rc_history ();
 #endif
 #if defined (DEBUG_MEMORY_LEAKAGE)
 		}
@@ -757,7 +750,7 @@ isc_result_t omapi_protocol_signal_handler (omapi_object_t *h,
 		dmalloc_dump_outstanding ();
 #endif
 #if defined (DEBUG_RC_HISTORY_EXHAUSTIVELY)
-		dump_rc_history (h);
+		dump_rc_history ();
 #endif
 #if defined (DEBUG_MEMORY_LEAKAGE)
 		previous_outstanding = 0xDEADBEEF;
