@@ -3,7 +3,7 @@
    Server-specific in-memory database support. */
 
 /*
- * Copyright (c) 2004-2005 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2007 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -34,7 +34,7 @@
 
 #ifndef lint
 static char copyright[] =
-"$Id: mdb.c,v 1.67.2.25 2005/10/10 16:56:47 dhankins Exp $ Copyright (c) 2004-2005 Internet Systems Consortium.  All rights reserved.\n";
+"$Id: mdb.c,v 1.67.2.31 2007/06/08 18:59:13 dhankins Exp $ Copyright (c) 2004-2005 Internet Systems Consortium.  All rights reserved.\n";
 #endif /* not lint */
 
 #include "dhcpd.h"
@@ -792,8 +792,6 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
 	int propogate;
 	int pimmediate;
 {
-	int enter_uid = 0;
-	int enter_hwaddr = 0;
 	struct lease *lp, **lq, *prev;
 	TIME lp_next_state;
 
@@ -840,28 +838,21 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
 
 	/* If there's a Unique ID, dissociate it from the hash
 	   table and free it if necessary. */
-	if (comp -> uid) {
-		uid_hash_delete (comp);
-		enter_uid = 1;
-		if (comp -> uid != &comp -> uid_buf [0]) {
-			dfree (comp -> uid, MDL);
-			comp -> uid_max = 0;
-			comp -> uid_len = 0;
+	if (comp->uid) {
+		uid_hash_delete(comp);
+		if (comp->uid != comp->uid_buf) {
+			dfree(comp->uid, MDL);
+			comp->uid_max = 0;
+			comp->uid_len = 0;
 		}
 		comp -> uid = (unsigned char *)0;
-	} else
-		enter_uid = 1;
+	}
 
-	if (comp -> hardware_addr.hlen &&
-	    ((comp -> hardware_addr.hlen !=
-	      lease -> hardware_addr.hlen) ||
-	     memcmp (comp -> hardware_addr.hbuf,
-		     lease -> hardware_addr.hbuf,
-		     comp -> hardware_addr.hlen))) {
-		hw_hash_delete (comp);
-		enter_hwaddr = 1;
-	} else if (!comp -> hardware_addr.hlen)
-		enter_hwaddr = 1;
+	/* If there's a hardware address, remove the lease from its
+	 * old position in the hash bucket's ordered list.
+	 */
+	if (comp->hardware_addr.hlen)
+		hw_hash_delete(comp);
 
 	/* If the lease has been billed to a class, remove the billing. */
 	if (comp -> billing_class != lease -> billing_class) {
@@ -951,14 +942,12 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
 	}
 
 	/* Record the lease in the uid hash if necessary. */
-	if (enter_uid && comp -> uid) {
-		uid_hash_add (comp);
-	}
+	if (comp->uid)
+		uid_hash_add(comp);
 
 	/* Record it in the hardware address hash if necessary. */
-	if (enter_hwaddr && lease -> hardware_addr.hlen) {
-		hw_hash_add (comp);
-	}
+	if (comp->hardware_addr.hlen)
+		hw_hash_add(comp);
 
 #if defined (FAILOVER_PROTOCOL)
 	comp->cltt = lease->cltt;
@@ -1031,14 +1020,13 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
 	}
 
 	if (!lp) {
-		log_error ("Lease with binding state %s not on its queue.",
-			   (comp -> binding_state < 1 ||
-			    comp -> binding_state > FTS_LAST)
-			   ? "unknown"
-			   : binding_state_names [comp -> binding_state - 1]);
-		return 0;
+		log_fatal("Lease with binding state %s not on its queue.",
+			  (comp->binding_state < 1 ||
+			   comp->binding_state > FTS_LAST)
+			  ? "unknown"
+			  : binding_state_names[comp->binding_state - 1]);
 	}
-	
+
 	if (prev) {
 		lease_dereference (&prev -> next, MDL);
 		if (comp -> next) {
@@ -1412,6 +1400,9 @@ void abandon_lease (lease, message)
 	const char *message;
 {
 	struct lease *lt = (struct lease *)0;
+#if defined (NSUPDATE)
+	ddns_removals (lease);
+#endif
 
 	if (!lease_copy (&lt, lease, MDL))
 		return;
@@ -1441,6 +1432,9 @@ void dissociate_lease (lease)
 	struct lease *lease;
 {
 	struct lease *lt = (struct lease *)0;
+#if defined (NSUPDATE)
+	ddns_removals (lease);
+#endif
 
 	if (!lease_copy (&lt, lease, MDL))
 		return;
@@ -1583,29 +1577,117 @@ int find_lease_by_hw_addr (struct lease **lp,
 				  hwaddr, hwlen, file, line);
 }
 
-/* Add the specified lease to the uid hash. */
-
-void uid_hash_add (lease)
-	struct lease *lease;
+/* If the lease is preferred over the candidate, return truth.  The
+ * 'cand' and 'lease' names are retained to read more clearly against
+ * the 'uid_hash_add' and 'hw_hash_add' functions (this is common logic
+ * to those two functions).
+ *
+ * 1) ACTIVE leases are preferred.  The active lease with
+ *    the longest lifetime is preferred over shortest.
+ * 2) "transitional states" are next, this time with the
+ *    most recent CLTT.
+ * 3) free/backup/etc states are next, again with CLTT.  In truth we
+ *    should never see reset leases for this.
+ * 4) Abandoned leases are always dead last.
+ */
+static isc_boolean_t
+client_lease_preferred(struct lease *cand, struct lease *lease)
 {
-	struct lease *head = (struct lease *)0;
-	struct lease *next = (struct lease *)0;
+	if (cand->binding_state == FTS_ACTIVE) {
+		if (lease->binding_state == FTS_ACTIVE &&
+		    lease->ends >= cand->ends)
+			return ISC_TRUE;
+	} else if (cand->binding_state == FTS_EXPIRED ||
+		   cand->binding_state == FTS_RELEASED) {
+		if (lease->binding_state == FTS_ACTIVE)
+			return ISC_TRUE;
 
+		if ((lease->binding_state == FTS_EXPIRED ||
+		     lease->binding_state == FTS_RELEASED) &&
+		    lease->cltt >= cand->cltt)
+			return ISC_TRUE;
+	} else if (cand->binding_state != FTS_ABANDONED) {
+		if (lease->binding_state == FTS_ACTIVE ||
+		    lease->binding_state == FTS_EXPIRED ||
+		    lease->binding_state == FTS_RELEASED)
+			return ISC_TRUE;
+
+		if (lease->binding_state != FTS_ABANDONED &&
+		    lease->cltt >= cand->cltt)
+			return ISC_TRUE;
+	} else /* (cand->binding_state == FTS_ABANDONED) */ {
+		if (lease->binding_state != FTS_ABANDONED ||
+		    lease->cltt >= cand->cltt)
+			return ISC_TRUE;
+	}
+
+	return ISC_FALSE;
+}
+
+/* Add the specified lease to the uid hash. */
+void
+uid_hash_add(struct lease *lease)
+{
+	struct lease *head = NULL;
+	struct lease *cand = NULL;
+	struct lease *prev = NULL;
+	struct lease *next = NULL;
 
 	/* If it's not in the hash, just add it. */
 	if (!find_lease_by_uid (&head, lease -> uid, lease -> uid_len, MDL))
 		lease_hash_add (lease_uid_hash, lease -> uid,
 				lease -> uid_len, lease, MDL);
 	else {
-		/* Otherwise, attach it to the end of the list. */
-		while (head -> n_uid) {
-			lease_reference (&next, head -> n_uid, MDL);
-			lease_dereference (&head, MDL);
-			lease_reference (&head, next, MDL);
-			lease_dereference (&next, MDL);
+		/* Otherwise, insert it into the list in order of its
+		 * preference for "resuming allocation to the client."
+		 *
+		 * Because we don't have control of the hash bucket index
+		 * directly, we have to remove and re-insert the client
+		 * id into the hash if we're inserting onto the head.
+		 */
+		lease_reference(&cand, head, MDL);
+		while (cand != NULL) {
+			if (client_lease_preferred(cand, lease))
+				break;
+
+			if (prev != NULL)
+				lease_dereference(&prev, MDL);
+			lease_reference(&prev, cand, MDL);
+
+			if (cand->n_uid != NULL)
+				lease_reference(&next, cand->n_uid, MDL);
+
+			lease_dereference(&cand, MDL);
+
+			if (next != NULL) {
+				lease_reference(&cand, next, MDL);
+				lease_dereference(&next, MDL);
+			}
 		}
-		lease_reference (&head -> n_uid, lease, MDL);
-		lease_dereference (&head, MDL);
+
+		/* If we want to insert 'before cand', and prev is NULL,
+		 * then it was the head of the list.  Assume that position.
+		 */
+		if (prev == NULL) {
+			lease_reference(&lease->n_uid, head, MDL);
+			lease_hash_delete(lease_uid_hash, lease->uid,
+					  lease->uid_len, MDL);
+			lease_hash_add(lease_uid_hash, lease->uid,
+				       lease->uid_len, lease, MDL);
+		} else /* (prev != NULL) */ {
+			if(prev->n_uid != NULL) {
+				lease_reference(&lease->n_uid, prev->n_uid,
+						MDL);
+				lease_dereference(&prev->n_uid, MDL);
+			}
+			lease_reference(&prev->n_uid, lease, MDL);
+
+			lease_dereference(&prev, MDL);
+		}
+
+		if (cand != NULL)
+			lease_dereference(&cand, MDL);
+		lease_dereference(&head, MDL);
 	}
 }
 
@@ -1659,11 +1741,13 @@ void uid_hash_delete (lease)
 
 /* Add the specified lease to the hardware address hash. */
 
-void hw_hash_add (lease)
-	struct lease *lease;
+void
+hw_hash_add(struct lease *lease)
 {
-	struct lease *head = (struct lease *)0;
-	struct lease *next = (struct lease *)0;
+	struct lease *head = NULL;
+	struct lease *cand = NULL;
+	struct lease *prev = NULL;
+	struct lease *next = NULL;
 
 	/* If it's not in the hash, just add it. */
 	if (!find_lease_by_hw_addr (&head, lease -> hardware_addr.hbuf,
@@ -1673,16 +1757,59 @@ void hw_hash_add (lease)
 				lease -> hardware_addr.hlen,
 				lease, MDL);
 	else {
-		/* Otherwise, attach it to the end of the list. */
-		while (head -> n_hw) {
-			lease_reference (&next, head -> n_hw, MDL);
-			lease_dereference (&head, MDL);
-			lease_reference (&head, next, MDL);
-			lease_dereference (&next, MDL);
+		/* Otherwise, insert it into the list in order of its
+		 * preference for "resuming allocation to the client."
+		 *
+		 * Because we don't have control of the hash bucket index
+		 * directly, we have to remove and re-insert the client
+		 * id into the hash if we're inserting onto the head.
+		 */
+		lease_reference(&cand, head, MDL);
+		while (cand != NULL) {
+			if (client_lease_preferred(cand, lease))
+				break;
+
+			if (prev != NULL)
+				lease_dereference(&prev, MDL);
+			lease_reference(&prev, cand, MDL);
+
+			if (cand->n_hw != NULL)
+				lease_reference(&next, cand->n_hw, MDL);
+
+			lease_dereference(&cand, MDL);
+
+			if (next != NULL) {
+				lease_reference(&cand, next, MDL);
+				lease_dereference(&next, MDL);
+			}
 		}
 
-		lease_reference (&head -> n_hw, lease, MDL);
-		lease_dereference (&head, MDL);
+		/* If we want to insert 'before cand', and prev is NULL,
+		 * then it was the head of the list.  Assume that position.
+		 */
+		if (prev == NULL) {
+			lease_reference(&lease->n_hw, head, MDL);
+			lease_hash_delete(lease_hw_addr_hash,
+					  lease->hardware_addr.hbuf,
+					  lease->hardware_addr.hlen, MDL);
+			lease_hash_add(lease_hw_addr_hash,
+				       lease->hardware_addr.hbuf,
+				       lease->hardware_addr.hlen,
+				       lease, MDL);
+		} else /* (prev != NULL) */ {
+			if(prev->n_hw != NULL) {
+				lease_reference(&lease->n_hw, prev->n_hw,
+						MDL);
+				lease_dereference(&prev->n_hw, MDL);
+			}
+			lease_reference(&prev->n_hw, lease, MDL);
+
+			lease_dereference(&prev, MDL);
+		}
+
+		if (cand != NULL)
+			lease_dereference(&cand, MDL);
+		lease_dereference(&head, MDL);
 	}
 }
 
@@ -2037,10 +2164,19 @@ void expire_all_pools ()
 		    for (l = *(lptr [i]); l; l = l -> next) {
 			p -> lease_count++;
 			if (l -> ends <= cur_time) {
-				if (l -> binding_state == FTS_FREE)
-					p -> free_leases++;
-				else if (l -> binding_state == FTS_BACKUP)
-					p -> backup_leases++;
+				if (l->binding_state == FTS_FREE) {
+					if (i == FREE_LEASES)
+						p->free_leases++;
+					else
+						log_fatal("Impossible case "
+							  "at %s:%d.", MDL);
+				} else if (l->binding_state == FTS_BACKUP) {
+					if (i == BACKUP_LEASES)
+						p->backup_leases++;
+					else
+						log_fatal("Impossible case "
+							  "at %s:%d.", MDL);
+				}
 			}
 #if defined (FAILOVER_PROTOCOL)
 			if (p -> failover_peer &&
